/* $Id: String.hh,v 1.2 2002/04/28 19:29:00 sdevaux Exp $
 *
 * NaJA
 * Tabs = 4 chars
 * copyright (C) 2002 Sebastien Devaux
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
 *
 *              Sebastien Devaux <sebastien.devaux@laposte.net>
 */ 

#ifndef __String_hh__
#define __String_hh__

#include "naja.hh"
#define MAX_INT 55555

PACKAGE_BEGIN(najaLang)

class String: public Object {
private:
	/** chaine vide par defaut */
	static char * null;
	/** longueur de la chaine */
	int len;
	/** Le contenu de la chaine proprement dit */
	char * value;
	/**
	 * Native c string length
	 * @param str string to monitor
	 * @return the lengh i.e. the count of char before the first \0 char
	 */ 
	int strlen(const char * str);
	/**
	 * Native string copy
	 * @param dest target string, enough memory must be allocated.
	 * @param src source string
	 * @param lmax max length to copy
	 */
	void strcpy(char * dest,const char * src,int lmax);
	/** 
	 * Native string compare
	 * @param s1 first string to compare
	 * @param s2 second string to compare
	 * @param lmax max length to compare (to full string comparaison, it must
	 *			include the shortest string's end char \0.
	 * @return <li>0 string are equals</li>
	 *		   <li>&gt;0 s1 > s2</li>
	 *		   <li>&lt;0 s1 < s2</li>
	 */
	int strcmp(const char * s1,const char * s2,int lmax);
	/** 
	 * Native string compare ignoring case
	 * @param s1 first string to compare
	 * @param s2 second string to compare
	 * @param lmax max length to compare (to full string comparaison, it must
	 *			include the shortest string's end char \0.
	 * @return <li>0 string are equals</li>
	 *		   <li>&gt;0 s1 > s2</li>
	 *		   <li>&lt;0 s1 < s2</li>
	 */
	int strcasecmp(const char * s1,const char * s2,int lmax);
	/**
	 * Create a new String from two Strings concatened
	 * @param str1 first string
	 * @param str2 second string
	 */
	String(const char * str1,const char * str2);
	/**
	 * Create a new String from another replacing one char with another
	 * @param str input String
	 * @param oldChar the old character
	 * @param newChar the new character
	 */
	String(NAJA<String> str,char oldChar,char newChar);
public:
	/**
     * Initializes a newly created String object so that it represents an empty
	 * character sequence.
	 */
	String();
	/**
	 * Free locally allocated memory
	 */
	~String();
	/** 
	 * Allocates a new String so that it represents the sequence of characters
	 * currently contained in the character array argument. The contents of the
     * character array are copied; subsequent modification of the character
	 * array does not affect the newly created string.
     * @param str the initial value of the string.
	 */
	String(const char * str);
	/**
	 * Allocates a new String that contains characters from a subarray of the
	 * character array argument. The offset argument is the index of the first
     * character of the subarray and the count argument specifies the length of
	 * the subarray. The contents of the subarray are copied; subsequent
     * modification of the character array does not affect the newly created string.
     * @param str array that is the source of characters.
     * @param offset the initial offset.
     * @param count the length.
	 */
	String(const char * str,int offset,int count);
	/**
	 * Returns the character at the specified index. An index ranges from 0 to
	 * length() - 1. The first character of the sequence is at index 0, the next
     * at index 1, and so on, as for array indexing.
	 * @param index the index of the character.
	 * @return the character at the specified index of this string. The first
	 *		 character is at index 0.
	 */
	virtual char charAt(int index);
	virtual int length();
	/**
	 * @see charAt
	 */
	virtual char operator[](int index);
	/**
	 * Copies characters from this string into the destination character array. 
     * The first character to be copied is at index srcBegin; the last character
	 * to be copied is at index srcEnd-1 (thus the total number of characters to
     * be copied is srcEnd-srcBegin). The characters are copied into the subarray
	 * of dst starting at index dstBegin and ending at index: 
     * <pre>dstBegin + (srcEnd-srcBegin) - 1</pre>
	 * @param srcBegin index of the first character in the string to copy.
     * @param srcEnd index after the last character in the string to copy.
     * @param dst the destination array.
     * @param dstBegin the start offset in the destination array.
	 */
	virtual void getChars(int srcBegin,int srcEnd,char * dst,int dstBegin);
	/**
	 * Compares this String to another String, ignoring case.
	 * @param anotherString the String to compare this String against.
	 * @return true if the argument is not null and the Strings are equal, ignoring 
	 * 		case; false otherwise.
	 */
	virtual boolean equalsIgnoreCase(NAJA<String> anotherString);
	/**
	 * Compares two strings.
     * @param anotherString the String to be compared.
     * @return the value 0 if the argument string is equal to this string; a value less
	 *      than 0 if this string is lexicographically less than the string argument;
     *      and a value greater than 0 if this string is lexicographically greater than
	 *		 the string argument.
	 */
	virtual int compareTo(NAJA<String> anotherString);
	/**
	 * Compares two strings, ignoring case considerations.
	 * @param anotherString the String to be compared.
	 * @return a negative integer, zero, or a positive integer as the the specified
	 * 		String is greater than, equal to, or less than this String, ignoring case
     *      considerations.
	 */
	virtual int compareToIgnoreCase(NAJA<String> anotherString);
	/**
	 * Tests if two string regions are equal. 
	 * @param toffset the starting offset of the subregion in this string.
	 * @param other the string argument.
	 * @param ooffset the starting offset of the subregion in the string argument.
	 * @param l the number of characters to compare.
	 */
	virtual boolean regionMatches(int toffset,NAJA<String> other,
													int ooffset,int l);
	/**
	 * Tests if two string regions are equal. 
	 */
	virtual boolean regionMatch(boolean ignoreCase,int toffset,
							NAJA<String> other,int ooffset,int l);
	/**
	 * Tests if this string starts with the specified prefix beginning a 
	 * specified index.
	 */
	virtual boolean startsWith(NAJA<String> prefix,int offset);
	/**
	 * Tests if this string starts with the specified prefix.
	 */
	virtual boolean startsWith(NAJA<String> prefix);
	/**
	 * Tests if this string ends with the specified suffix.
	 */
	virtual boolean endsWith(NAJA<String> suffix);		
	/**
	 * Returns a hashcode for this string. The hashcode for a String object
	 * is computed as :
	 * <pre>s[0]*31^(n-1) + s[1]*31^(n-2) + ... + s[n-1]</pre>
	 */
	virtual int hashCode();
    /**
     * Returns the index within this string of the first occurrence of the
     * specified character, starting the search at the specified index.
     * @param   ch          a character.
     * @param   fromIndex   the index to start the search from.
     * @return  the index of the first occurrence of the character in the
     *          character sequence represented by this object that is greater
     *          than or equal to <code>fromIndex</code>, or <code>-1</code>
     *          if the character does not occur.
     */
    virtual int indexOf(int ch, int fromIndex=0);
	/**
     * Returns the index within this string of the last occurrence of the
     * specified character, searching backward starting at the specified 
     * index.
     * @param   ch          a character.
     * @param   fromIndex   the index to start the search from. There is no 
     *          restriction on the value of <code>fromIndex</code>. If it is 
     *          greater than or equal to the length of this string, it has 
     *          the same effect as if it were equal to one less than the 
     *          length of this string: this entire string may be searched. 
     *          If it is negative, it has the same effect as if it were -1: 
     *          -1 is returned.
     * @return  the index of the last occurrence of the character in the
     *          character sequence represented by this object that is less
     *          than or equal to <code>fromIndex</code>, or <code>-1</code>
     *          if the character does not occur before that point.
     */
    virtual int lastIndexOf(int ch, int fromIndex=MAX_INT);
	/**
	 * Returns the index within this string of the first occurrence of the
     * specified substring, starting at the specified index.
     * @param   str         the substring to search for.
     * @param   fromIndex   the index to start the search from.
     * @return  If the string argument occurs as a substring within this
     *          object at a starting index no smaller than
     *          <code>fromIndex</code>, then the index of the first character
     *          of the first such substring is returned. If it does not occur
     *          as a substring starting at <code>fromIndex</code> or beyond,
     *          <code>-1</code> is returned.
     * @exception java.lang.NullPointerException if <code>str</code> is 
     *          <code>null</code>
     */
	virtual int indexOf(NAJA<String> str, int fromIndex=0);
    /**
     * Returns the index within this string of the last occurrence of
     * the specified substring.
     * @param   str         the substring to search for.
     * @param   fromIndex   the index to start the search from. There is no 
     *          restriction on the value of fromIndex. If it is greater than 
     *          the length of this string, it has the same effect as if it 
     *          were equal to the length of this string: this entire string 
     *          may be searched. If it is negative, it has the same effect 
     *          as if it were -1: -1 is returned.
     * @return  If the string argument occurs one or more times as a substring
     *          within this object at a starting index no greater than
     *          <code>fromIndex</code>, then the index of the first character of
     *          the last such substring is returned. If it does not occur as a
     *          substring starting at <code>fromIndex</code> or earlier,
     *          <code>-1</code> is returned.
     */
    virtual int lastIndexOf(NAJA<String> str, int fromIndex=MAX_INT);
    /**
     * Returns a new string that is a substring of this string.
     * @param      beginIndex   the beginning index, inclusive.
     * @param      endIndex     the ending index, exclusive.
     * @return     the specified substring.
     */
    virtual NAJA<String> substring(int beginIndex, int endIndex=MAX_INT);
	/**
     * Concatenates the specified string to the end of this string. 
     * @param   str   the <code>String</code> that is concatenated to the end
     *                of this <code>String</code>.
     * @return  a string that represents the concatenation of this object's
     *          characters followed by the string argument's characters.
     */
    virtual NAJA<String> concat(NAJA<String> str);
	/**
     * Returns a new string resulting from replacing all occurrences of 
     * <code>oldChar</code> in this string with <code>newChar</code>. 
     * @param   oldChar   the old character.
     * @param   newChar   the new character.
     * @return  a string derived from this string by replacing every
     *          occurrence of <code>oldChar</code> with <code>newChar</code>.
     */
    virtual NAJA<String> replace(char oldChar, char newChar);
	/**
     * Removes white space from both ends of this string. 
     * @return  this string, with white space removed from the front and end.
     */
    virtual NAJA<String> trim();
	//---------------------------------------------------------
	// Object overrides
	/** 
	 * toString() override
	 * @return this string itself.
	 */
	virtual NAJA<String> toString();
	/**
	 * Compares this string to the specified object. The result is true if and only
     * if the argument is not null and is a String object that represents the
     * same sequence of characters as this object.
	 * @param anotherString the object to compare this String against.
	 * @return true if the String are equal; false otherwise.
	 */
	virtual boolean equals(NAJA<String> anotherString);  
	/**
	 * clone() override
	 * @return a copy of this string
	 */
	virtual NAJA<Object> clone();
	//---------------------------------------------------------
	// CCJLIB extensions
	/**
     * get the internal native c-string
	 */
	virtual const char * c_str();
	//------------------------------------------------------
	friend NAJA<String> operator+(NAJA<String> str1,NAJA<String> str2);
	friend NAJA<String> operator+(NAJA<String> str1,const char * str2);
	friend NAJA<String> operator+(const char * str1,NAJA<String> str2);
};

extern "C++" {
NAJA<String> operator+(NAJA<String> str1,NAJA<String> str2);
NAJA<String> operator+(NAJA<String> str1,const char * str2);
NAJA<String> operator+(const char *,NAJA<String> str2);
}

PACKAGE_END

#endif // __String_hh__
